from django.core.management import BaseCommand
from sqlalchemy_schemadisplay import create_uml_graph

from vox_django.models import models


class Command(BaseCommand):
    help = 'Generates uml from models'

    def handle(self, *args, **kwargs):
        graph = create_uml_graph(models,
                                 show_operations=False,
                                 show_multiplicity_one=False)
        graph.write_png('schema.png')



from sqlalchemy.orm import class_mapper
from sqlalchemy_pagination import paginate

models = []


def uml_model(model):
    models.append(class_mapper(model))
    return model


class Paginator:
    def __init__(self, query, current_page, page_size=10):
        self.current = None
        self.current_page = current_page
        self.query = query
        self.page_size = page_size
        self.max_pages = None

    def __iter__(self):
        return self

    def __next__(self):
        if self.max_pages is not None and self.page_number > self.max_pages:
            raise StopIteration

        if self.current is None:
            self.load_page()

            return self.current

        if self.current.has_next:
            self.current_page = self.current.next_page
            self.load_page()

            return self.current

        raise StopIteration

    def load_page(self):
        self.current = paginate(self.query, self.current_page, self.page_size)

    @property
    def items(self):
        return self.current.items

    @property
    def page_number(self):
        return self.current_page



import logging

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin
from easy_profile import SessionProfiler, StreamReporter

profiler = SessionProfiler()


class SqlAlchemyProfilerMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if settings.DEBUG:
            profiler.begin()

            response = self.get_response(request)

            profiler.commit()

            StreamReporter().report(f'{request.method} {request.path}', profiler.stats)

            return response

        return self.get_response(request)


class ExceptionMiddleware(MiddlewareMixin):
    def process_exception(self, request, error):
        logging.getLogger('django').error(error, extra={"appName": settings.APP_NAME,
                                                        "appVersion": "1.0.0"})



from django.test.runner import DiscoverRunner
from django_sorcery.db import databases


class TestRunner(DiscoverRunner):
    def __init__(self, **kwargs):
        super(TestRunner, self).__init__(**kwargs)
        self.aliases = []

    def setup_databases(self, **kwargs):
        for alias in kwargs['aliases']:
            self.aliases.append(alias)
            databases.get(alias).create_all()

    def teardown_databases(self, old_config, **kwargs):
        for alias in self.aliases:
            databases.get(alias).drop_all()



from django_sorcery.db import databases
from health_check.backends import BaseHealthCheckBackend
from health_check.exceptions import HealthCheckException

db = databases.get("default")


class DatabaseHealthCheck(BaseHealthCheckBackend):
    critical_service = True

    def check_status(self):
        try:
            return db.execute('SELECT 1 FROM DUAL')
        except Exception as e:
            raise HealthCheckException(str(e))

    def identifier(self):
        return 'Database'



from unittest import TestCase
from unittest.mock import patch, call, MagicMock

from django.conf import settings
from health_check.exceptions import HealthCheckException
from sqlalchemy_pagination import Page

from vox_django.criptografia import decrypt_aes, encrypt_aes

settings.configure(
    DEBUG=True,
    DATABASES={'default': {
        'ENGINE': 'sqlite'
    }},
    APP_NAME='foo',
    APP_VERSION='1.0',
    HEALTH_CHECK_PLUGINS=[
        'database',
    ],
    INSTALLED_APPS=[
        'vox_django',
    ],
    ORGANIZATION='riachuelo',
)


class TestCriptography(TestCase):
    def test_should_decrypt(self):
        PASS = 'j7ysUJr7buPaE7jNuNyI7g=='

        self.assertEqual(decrypt_aes(PASS), 'simula')

    def test_should_encrypt(self):
        PASS = 'simula'

        self.assertEqual(encrypt_aes(PASS), 'j7ysUJr7buPaE7jNuNyI7g==')


class TestHealthCheckBackends(TestCase):
    @patch('vox_django.healthcheck.backends.db')
    def test_should_healthcheck_database(self, db):
        from vox_django.healthcheck.backends import DatabaseHealthCheck

        DatabaseHealthCheck().check_status()
        db.execute.assert_called()

        db.execute.side_effect = Exception('error')

        with self.assertRaises(HealthCheckException):
            DatabaseHealthCheck().check_status()


class TestMiddleware(TestCase):
    @patch('vox_django.middleware.logging.getLogger')
    def test_should_log_error(self, getLogger):
        from vox_django.middleware import ExceptionMiddleware

        middlwware = ExceptionMiddleware()
        middlwware.process_exception(request={}, error='some error')

        getLogger().error.assert_called_with('some error', extra={"appName": settings.APP_NAME,
                                                                  "appVersion": "1.0.0"})


class TestModels(TestCase):
    @patch('vox_django.models.paginate')
    def test_should_paginate(self, paginate):
        paginate.side_effect = None
        paginate.return_value = Page(items=[], page=1, page_size=10, total=100)
        from vox_django.models import Paginator
        paginator = Paginator('select', 1)

        paginator.__next__()

        self.assertEqual([], paginator.items)
        self.assertEqual(1, paginator.current_page)

    @patch('vox_django.models.paginate')
    def test_should_iterate_pages(self, paginate):
        from vox_django.models import Paginator

        pages_stub = [
            Page(items=[{'test': 1}], page=1, page_size=1, total=2),
            Page(items=[{'test': 2}], page=2, page_size=1, total=2),
        ]

        paginate.side_effect = pages_stub

        pages = Paginator('select', 1)

        cur_page = 0
        for page in pages:
            cur_page += 1
            self.assertEqual(pages.page_number, cur_page)
            self.assertTrue(page in pages_stub)


class TestDjangoApp(TestCase):
    @patch('vox_django.apps.plugin_dir')
    def test_should_install_health_check_plugins(self, plugin_dir):
        from vox_django.apps import VoxDjangoConfig
        from vox_django.healthcheck.backends import DatabaseHealthCheck

        VoxDjangoConfig.path = 'vox_django'
        config = VoxDjangoConfig('foo', 'vox_django')
        config.ready()

        plugin_dir.register.assert_has_calls([
            call(DatabaseHealthCheck),
        ])



from base64 import b64encode, b64decode

from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad

BS = 16
key = bytearray([45, 42, 45, 66, 45, 73, 76, 68, 67, 42, 79, 68, 69, 45, 45, 45])
cipher = AES.new(key, AES.MODE_ECB)


def encrypt_aes(message):
    return b64encode(cipher.encrypt(pad(message.encode(), 16))).decode('utf8')


def decrypt_aes(encoded):
    data = cipher.decrypt(b64decode(encoded))

    return unpad(data, 16).decode('utf8')



from django.apps import AppConfig
from django.conf import settings
from health_check.plugins import plugin_dir

from vox_django.healthcheck.backends import DatabaseHealthCheck

HEALTH_CHECK_PLUGINS = settings.HEALTH_CHECK_PLUGINS

AVAILABLE_PLUGINS = {
    'database': DatabaseHealthCheck,
}


class VoxDjangoConfig(AppConfig):
    name = 'vox_django'

    def ready(self):
        [plugin_dir.register(AVAILABLE_PLUGINS[plugin]) for plugin in HEALTH_CHECK_PLUGINS]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vox_django',
 'vox_django.healthcheck',
 'vox_django.management',
 'vox_django.management.commands']

package_data = \
{'': ['*']}

install_requires = \
['django-cors-headers',
 'django-health-check',
 'django-rest-witchcraft',
 'django-sorcery',
 'django>=3.0,<3.1',
 'djangorestframework>=3.11,<3.12',
 'drf-yasg',
 'pycryptodome',
 'python-dotenv',
 'sqlalchemy-easy-profile>=1.2.0,<2.0.0',
 'sqlalchemy-pagination',
 'sqlalchemy-schemadisplay',
 'sqlalchemy==1.3.21']

setup_kwargs = {
    'name': 'vox-django',
    'version': '0.1.1',
    'description': '',
    'long_description': None,
    'author': 'Jhonatan Teixeira',
    'author_email': 'jhonatan.teixeira@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
